<?php

namespace Drupal\email_tfa\EventSubscriber;

use Drupal\Core\TempStore\PrivateTempStoreFactory;
use Drupal\Core\Routing\RouteMatchInterface;
use Drupal\Core\Session\AccountInterface;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Symfony\Component\HttpKernel\Event\GetResponseEvent;
use Symfony\Component\HttpKernel\KernelEvents;
use Drupal\Core\Url;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Messenger\MessengerInterface;

/**
 * Redirect user to Email TFA.
 */
class EmailTfaSubscriber implements EventSubscriberInterface {

  /**
   * The current user.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  protected $account;

  /**
   * The route match object for the current page.
   *
   * @var \Drupal\Core\Routing\RouteMatchInterface
   */
  protected $routeMatch;


  /**
   * The aggregator.settings config object.
   *
   * @var \Drupal\Core\Config\Config
   */
  protected $config;


  /**
   * The tempstore factory.
   *
   * @var \Drupal\Core\TempStore\PrivateTempStoreFactory
   */
  protected $tempStoreFactory;

  /**
   * The messenger.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;

  /**
   * Constructs a new subscriber after login.
   *
   * @param \Drupal\Core\Session\AccountInterface $account
   *   The current user.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   * @param \Drupal\Core\Routing\RouteMatchInterface $route_match
   *   The Route Match.
   * @param \Drupal\Core\TempStore\PrivateTempStoreFactory $temp_store_factory
   *   The factory for the temp store object.
   * @param \Drupal\Core\Messenger\MessengerInterface $messenger
   *   The messenger.
   */
  public function __construct(AccountInterface $account, ConfigFactoryInterface $config_factory, RouteMatchInterface $route_match, PrivateTempStoreFactory $temp_store_factory, MessengerInterface $messenger) {
    $this->account = $account;
    $this->config = $config_factory->get('email_tfa.settings');
    $this->routeMatch = $route_match;
    $this->tempStoreFactory = $temp_store_factory->get('email_tfa');
    $this->messenger = $messenger;
  }

  /**
   * Listen to kernel.request events and call emailTfaRedirection.
   */
  public static function getSubscribedEvents() {
    $events[KernelEvents::REQUEST][] = ['emailTfaRedirection', 30];
    return $events;
  }

  /**
   * Redirect pattern based url.
   *
   * @param \Symfony\Component\HttpKernel\Event\GetResponseEvent $event
   *   Response event.
   */
  public function emailTfaRedirection(GetResponseEvent $event) {
    $routes = $this->config->get('routes');
    $routes = array_map('trim', explode("\n", $routes));
    $status = $this->config->get('status');
    $timeouts = $this->config->get('timeouts');
    // Check if module is active from its settings.
    if ($status) {
      // Check if user is NOT already verified && is Authenticated.
      if ($this->tempStoreFactory->get('email_tfa_user_verify') == 0 && $this->account->isAuthenticated()) {
        // If time is passed/expired then redirects user to login page.
        if (time() - $this->tempStoreFactory->get('email_tfa_mail_created') > $timeouts) {
          user_logout();
          $url = Url::fromRoute('user.page')->toString();
          $response = new RedirectResponse($url);
          $response->send();
          exit;
        }
        // Pages that excluded from the redirect from module settings.
        if (!in_array($this->routeMatch->getRouteName(), $routes)) {
          $url = Url::fromRoute('email_tfa.verifiy')->toString();
          $response = new RedirectResponse($url);
          $response->send();
          exit;
        }
      }
    }
  }

}
